@@ -4,6 +4,7 @@ import boto3
 import io
 import json
 import sqlite_utils
+import time
 
 S3_OCR_JSON = ".s3-ocr.json"
 
@@ -44,7 +45,9 @@ def common_boto3_options(fn):
     return fn
 
 
-def make_client(service, access_key, secret_key, session_token, endpoint_url, auth):
+def make_client(
+    service, access_key, secret_key, session_token, endpoint_url, auth, region_name=None
+):
     if auth:
         if access_key or secret_key or session_token:
             raise click.ClickException(
@@ -77,6 +80,8 @@ def make_client(service, access_key, secret_key, session_token, endpoint_url, au
         kwargs["aws_session_token"] = session_token
     if endpoint_url:
         kwargs["endpoint_url"] = endpoint_url
+    if region_name:
+        kwargs["region_name"] = region_name
     return boto3.client(service, **kwargs)
 
 
@@ -91,8 +96,12 @@ def cli():
 @click.argument("keys", nargs=-1)
 @click.option("--all", is_flag=True, help="Process all PDF files in the bucket")
 @click.option("--prefix", help="Process all PDF files within this prefix")
+@click.option(
+    "--dry-run", is_flag=True, help="Show what this would do, but don't actually do it"
+)
+@click.option("--no-retry", is_flag=True, help="Don't retry failed requests")
 @common_boto3_options
-def start(bucket, keys, all, prefix, **boto_options):
+def start(bucket, keys, all, prefix, dry_run, no_retry, **boto_options):
     """
     Start OCR tasks for PDF files in an S3 bucket
 
@@ -107,7 +116,8 @@ def start(bucket, keys, all, prefix, **boto_options):
         s3-ocr start name-of-bucket --prefix PUBLIC/
     """
     s3 = make_client("s3", **boto_options)
-    textract = make_client("textract", **boto_options)
+    bucket_region = s3.get_bucket_location(Bucket=bucket)["LocationConstraint"]
+    textract = make_client("textract", region_name=bucket_region, **boto_options)
     if keys:
         items = []
         for key in keys:
@@ -139,21 +149,41 @@ def start(bucket, keys, all, prefix, **boto_options):
             len(keys_with_s3_ocr_files), S3_OCR_JSON, len(pdf_items)
         )
     )
+    if dry_run:
+        items = [
+            item for item in pdf_items if item["Key"] not in keys_with_s3_ocr_files
+        ]
+        click.echo("Would start {} tasks for these keys:".format(len(items)))
+        for item in items:
+            click.echo(item["Key"])
+        return
     for item in pdf_items:
         key = item["Key"]
         if key not in keys_with_s3_ocr_files:
-            response = textract.start_document_text_detection(
-                DocumentLocation={
-                    "S3Object": {
-                        "Bucket": bucket,
-                        "Name": key,
-                    }
-                },
-                OutputConfig={
-                    "S3Bucket": bucket,
-                    "S3Prefix": "textract-output",
-                },
-            )
+            sleep = 1
+            while True:
+                try:
+                    response = start_document_text_extraction(
+                        textract,
+                        DocumentLocation={
+                            "S3Object": {
+                                "Bucket": bucket,
+                                "Name": key,
+                            }
+                        },
+                        OutputConfig={
+                            "S3Bucket": bucket,
+                            "S3Prefix": "textract-output",
+                        },
+                    )
+                    break
+                except textract.exceptions.LimitExceededException as ex:
+                    if no_retry:
+                        raise click.ClickException(str(ex))
+                    click.echo("{} - retrying...".format(str(ex)))
+                    time.sleep(sleep)
+                    if sleep < 8:
+                        sleep *= 2
             job_id = response.get("JobId")
             if job_id:
                 click.echo(f"Starting OCR for {key}, Job ID: {job_id}")
@@ -517,3 +547,8 @@ def paginate(service, method, list_key, **kwargs):
     paginator = service.get_paginator(method)
     for response in paginator.paginate(**kwargs):
         yield from response.get(list_key) or []
+
+
+def start_document_text_extraction(textract, **kwargs):
+    # Wrapper function to make this easier to mock in tests
+    return textract.start_document_text_detection(**kwargs)
